import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.border.*;

import java.io.File;

import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * ImageViewer is the main class of the image viewer application. It builds and
 * displays the application GUI and initialises all other components.
 * 
 * To start the application, create an object of this class.
 * 
 * @author Michael Kolling and David J Barnes 
 * @version 2.0
 */
public class ImageViewer
{
    // constants:
    private static final String VERSION = "Version 2.0";
    private static JFileChooser fileChooser = new JFileChooser(System.getProperty("user.dir"));

    // fields:
    private JFrame frame;
    private ImagePanel imagePanel;
    private JLabel filenameLabel;
    private JLabel statusLabel;
    JButton smallerButton, largerButton;
    private OFImage currentImage;
    
    private List filters;
    
    /**
     * Create an ImageViewer show it on screen.
     */
    public ImageViewer()
    {
        currentImage = null;
        filters = createFilters();
        makeFrame();
    }

    public static void main( String args[] )
    {
        new ImageViewer();
    }

    // ---- implementation of menu functions ----
    
    /**
     * Open function: open a file chooser to select a new image file.
     */
    private void openFile()
    {
        int returnVal = fileChooser.showOpenDialog(frame);

        if(returnVal != JFileChooser.APPROVE_OPTION) {
            return;  // cancelled
        }
        File selectedFile = fileChooser.getSelectedFile();
        currentImage = ImageFileManager.loadImage(selectedFile);

        if(currentImage == null) {   // image file was not a valid image
            JOptionPane.showMessageDialog(frame,
                    "The file was not in a recognized image file format.",
                    "Image Load Error",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        imagePanel.setImage(currentImage);
        showFilename(selectedFile.getPath());
        showStatus("File loaded.");
        smallerButton.setEnabled( true );
        largerButton.setEnabled( true );
        frame.pack();
    }
    
    /**
     * Close function: clase the current image.
     */
    private void close()
    {
        currentImage = null;
        imagePanel.clearImage();
        showFilename(null);
        largerButton.setEnabled( false );
        smallerButton.setEnabled( false );
    }
    
    /**
     * Quit function: quit the application.
     */
    private void quit()
    {
        System.exit(0);
    }
    
    
    /**
     * Apply a given filter to the current image.
     */
    private void applyFilter(Filter filter)
    {
        if(currentImage != null) {
            filter.apply(currentImage);
            frame.repaint();
            showStatus("Applied: " + filter.getName());
        }
        else {
            showStatus("No image loaded.");
        }
    }
    
    private void makeLarger()
    {
        int height = currentImage.getHeight();
        int width = currentImage.getWidth();
        
        OFImage newImage = new OFImage( 2*width, 2*height );
                
        for(int y = 0; y < height; y++) {
            for(int x = 0; x < width; x++) {
                Color c = currentImage.getPixel( x, y );
                newImage.setPixel( 2*x, 2*y, c );
                newImage.setPixel( 2*x + 1, 2*y, c );
                newImage.setPixel( 2*x, 2*y + 1, c );
                newImage.setPixel( 2*x + 1, 2*y + 1, c );
            }
        }
        
        currentImage = newImage;
        imagePanel.setImage(currentImage);
        frame.pack();
    }
    
    private void makeSmaller()
    {
        int height = currentImage.getHeight();
        int width = currentImage.getWidth();
        
        OFImage newImage = new OFImage( width/2, height/2 );
                
        // Sample some pixels
        for(int y = 0; y < height/2; y++) {
            for(int x = 0; x < width/2; x++) {
                Color c = currentImage.getPixel( 2*x, 2*y );
                newImage.setPixel( x, y, c );
            }
        }
        
        currentImage = newImage;
        imagePanel.setImage(currentImage);
        frame.pack();
    }
        
    /**
     * 'Lighter' function: make the picture lighter
     */
    private void showAbout()
    {
        JOptionPane.showMessageDialog(frame, 
                    "ImageViewer\n" + VERSION,
                    "About ImageViewer", 
                    JOptionPane.INFORMATION_MESSAGE);
    }
    
    
    // ---- support methods ----

    /**
     * Display a file name on the appropriate label.
     */
    private void showFilename(String filename)
    {
        if(filename == null) {
            filenameLabel.setText("No file displayed.");
        }
        else {
            filenameLabel.setText("File: " + filename);
        }
    }
    
    /**
     * Display a status message in the frame's status bar.
     */
    private void showStatus(String text)
    {
        statusLabel.setText(text);
    }
    
    /**
     * Create and return a list with all the known filters.
     */
    private List createFilters()
    {
        List filterList = new ArrayList();
        filterList.add(new DarkerFilter("Darker"));
        filterList.add(new LighterFilter("Lighter"));
        filterList.add(new ThresholdFilter("Threshold"));
        filterList.add(new SolarizationFilter("Solarize"));
        filterList.add(new InvertFilter("Invert"));
        filterList.add(new GrayscaleFilter("Grayscale"));
        filterList.add(new MirrorFilter("Mirror"));
                
        return filterList;
    }
    
    // ---- swing stuff to build the frame and all its components ----
    
    /**
     * Create the Swing frame and its content.
     */
    private void makeFrame()
    {
        frame = new JFrame("ImageViewer");
        JPanel contentPane = (JPanel)frame.getContentPane();
        contentPane.setBorder( new EmptyBorder( 6, 6, 6, 6 ) );
        
        // Specify the layout manager with nice spacing
        contentPane.setLayout(new BorderLayout(6, 6));
        
        makeMenuBar(frame);
         
        filenameLabel = new JLabel();
        contentPane.add(filenameLabel, BorderLayout.NORTH);

        imagePanel = new ImagePanel();
        imagePanel.setBorder( new EtchedBorder() );
        contentPane.add(imagePanel, BorderLayout.CENTER);
        
        makeToolbar( contentPane );

        statusLabel = new JLabel(VERSION);
        contentPane.add(statusLabel, BorderLayout.SOUTH);
        
        // building is done - arrange the components and show        
        showFilename(null);
        frame.pack();
        
        // center the frame on screen
        Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
        frame.setLocation(d.width/2 - frame.getWidth()/2, d.height/2 - frame.getHeight()/2);
        frame.setVisible(true);
    }
    
    /**
     * Create the main frame's menu bar.
     */
    private void makeMenuBar(JFrame frame)
    {
        final int SHORTCUT_MASK =
            Toolkit.getDefaultToolkit().getMenuShortcutKeyMask();


        JMenuBar menubar = new JMenuBar();
        frame.setJMenuBar(menubar);
        
        JMenu menu;
        JMenuItem item;
        
        // create the File menu
        menu = new JMenu("File");
        menubar.add(menu);
        
        item = new JMenuItem("Open");
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, SHORTCUT_MASK));
            item.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { openFile(); }
                           });
        menu.add(item);

        item = new JMenuItem("Close");
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, SHORTCUT_MASK));
            item.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { close(); }
                           });
        menu.add(item);
        menu.addSeparator();
        
        item = new JMenuItem("Quit");
            item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, SHORTCUT_MASK));
            item.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { quit(); }
                           });
        menu.add(item);


        // create the Filter menu
        menu = new JMenu("Filter");
        menubar.add(menu);
        
        for(Iterator it=filters.iterator(); it.hasNext(); ) {
            final Filter filter = (Filter)it.next();
            
            item = new JMenuItem(filter.getName());
            item.addActionListener(new ActionListener() {
                                public void actionPerformed(ActionEvent e) { 
                                    applyFilter(filter);
                                }
                           });
             menu.add(item);
         }

        // put a spacer into the menubar, so the next menu appears to the right
        menubar.add(Box.createHorizontalGlue());

        // create the Help menu
        menu = new JMenu("Help");
        menubar.add(menu);
        
        item = new JMenuItem("About ImageViewer...");
            item.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { showAbout(); }
                           });
        menu.add(item);

    }
    
    private void makeToolbar( Container contentPane )
    {
        JPanel toolbar = new JPanel();
        toolbar.setLayout( new GridLayout( 0, 1 ) );
        
        smallerButton = new JButton( "Smaller" );
        toolbar.add( smallerButton );
        smallerButton.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { makeSmaller(); }
                           });
        smallerButton.setEnabled( false );
        
        largerButton = new JButton( "Larger" );
        toolbar.add( largerButton );
        largerButton.addActionListener(new ActionListener() {
                               public void actionPerformed(ActionEvent e) { makeLarger(); }
                           });
        largerButton.setEnabled( false );
        
        JPanel flow = new JPanel();
        flow.add( toolbar );
        
        contentPane.add( flow, BorderLayout.WEST );
    }
}
